@@ -57,7 +57,10 @@
 #include <glib/gi18n-lib.h>
 
 #include <gdk/gdk.h>
+
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
+#endif
 
 #include "uniquebackend.h"
 #include "uniqueapp.h"
@@ -149,6 +152,7 @@ enum
 
 static guint unique_app_signals[LAST_SIGNAL] = { 0, };
 
+#ifdef GDK_WINDOWING_X11
 /* taken from nautilus */
 static guint32
 slowly_and_stupidly_obtain_timestamp (GdkDisplay *display)
@@ -195,6 +199,20 @@ slowly_and_stupidly_obtain_timestamp (GdkDisplay *display)
   return event.xproperty.time;
 }
 
+#else /* !GDK_WINDOWING_X11 */
+
+static guint32
+slowly_and_stupidly_obtain_timestamp (GdkDisplay *display G_GNUC_UNUSED)
+{
+  GTimeVal timeval = { 0, };
+
+  g_get_current_time (&timeval);
+
+  return timeval.tv_sec;
+}
+
+#endif /* GDK_WINDOWING_X11 */
+
 static void
 set_startup_id (UniqueBackend *backend,
                 const gchar   *startup_id)
@@ -209,8 +227,10 @@ set_startup_id (UniqueBackend *backend,
 
       /* try and get the startup notification id from GDK, the environment
        * or, if everything else failed, fake one.
-      */
+       */
+#ifdef GDK_WINDOWING_X11
       startup_id = gdk_x11_display_get_startup_notification_id (display);
+#endif /* GDK_WINDOWING_X11 */
 
       if (!startup_id || startup_id[0] == '\0')
         startup_id = g_getenv ("DESKTOP_STARTUP_ID");
@@ -271,6 +291,16 @@ unique_app_real_message_received (UniqueApp         *app,
   return UNIQUE_RESPONSE_OK;
 }
 
+static void
+window_weak_ref_cb (gpointer  user_data,
+                    GObject  *dead_object)
+{
+  UniqueApp *app = user_data;
+  UniqueAppPrivate *priv = app->priv;
+
+  priv->windows = g_slist_remove (priv->windows, dead_object);
+}
+
 static GObject *
 unique_app_constructor (GType                  gtype,
                         guint                  n_params,
@@ -371,6 +401,7 @@ unique_app_finalize (GObject *gobject)
 {
   UniqueApp *app = UNIQUE_APP (gobject);
   UniqueAppPrivate *priv = app->priv;
+  GSList *l;
 
   if (priv->commands_by_name)
     g_hash_table_destroy (priv->commands_by_name);
@@ -378,8 +409,10 @@ unique_app_finalize (GObject *gobject)
   if (priv->commands_by_id)
     g_hash_table_destroy (priv->commands_by_id);
 
-  if (priv->windows)
-    g_slist_free (priv->windows);
+  for (l = priv->windows; l != NULL; l = l->next)
+    g_object_weak_unref (l->data, window_weak_ref_cb, app);
+
+  g_slist_free (priv->windows);
 
   G_OBJECT_CLASS (unique_app_parent_class)->finalize (gobject);
 }
@@ -767,16 +800,6 @@ unique_app_add_command (UniqueApp   *app,
                         command_nick);
 }
 
-static void
-window_weak_ref_cb (gpointer  user_data,
-                    GObject  *object)
-{
-  UniqueApp *app = user_data;
-  UniqueAppPrivate *priv = app->priv;
-
-  priv->windows = g_slist_remove (priv->windows, object);
-}
-
 /**
  * unique_app_watch_window:
  * @app: a #UniqueApp
